/*
Copyright 2012 Helpers
https://github.com/provirus/Helpers

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package ca.pgon.helpers;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Stack;

import org.apache.commons.lang3.StringUtils;

import ca.pgon.helpers.filefilters.DirectoriesFileFilter;

/**
 * Some common functions to help with directories.
 * 
 * @author Simon Levesque
 * 
 */
public class DirectoryHelper {

	/**
	 * Remove the . and .. from a path. Must not have the root inside (e.g
	 * "tmp/dir/../file" not "/tmp/dir/../file")
	 * 
	 * @param path
	 *            the path
	 * @return the path cleaned up
	 */
	public static String cleanupDots(String path) {
		Stack<String> stack = new Stack<String>();

		for (String part : path.split("/")) {
			if (StringUtils.equals(part, ".")) {
				// Skip
				continue;
			}

			if (StringUtils.equals(part, "..")) {
				// Remove
				if (!stack.isEmpty()) {
					stack.pop();
				}
			} else {
				// Add
				stack.push(part);
			}
		}

		return StringUtils.join(stack, "/");
	}

	/**
	 * Creates all the parents directories of the destination file.
	 * 
	 * @param destination
	 *            the file that needs to have all its parents directories
	 *            created
	 * @return true if it succeeded
	 */
	public static boolean createDirectoriesToFile(File destination) {
		String fullPath = destination.getAbsolutePath();

		// Get the last separator
		int backSlashPos = fullPath.lastIndexOf('\\');
		int forwardSlashPos = fullPath.lastIndexOf('/');
		int position = Math.max(backSlashPos, forwardSlashPos);
		if (position == -1) {
			// Not a valid path
			return false;
		}

		// Get the directories path
		String directoriesPath = fullPath.substring(0, position);

		return createDirectory(directoriesPath);
	}

	/**
	 * Create a directory with all its parents.
	 * 
	 * @param directoryPath
	 *            the absolute path
	 * @return true if successfully created
	 */
	public static boolean createDirectory(String directoryPath) {
		File directory = new File(directoryPath);
		return directory.exists() || directory.mkdirs();
	}

	/**
	 * Create a temporary directory.
	 * 
	 * @param prefix
	 *            The prefix string to be used in generating the dir's name;
	 *            must be at least three characters long
	 * @return the directory or null if could not create
	 */
	public static File createTemporaryDirectory(String prefix) {
		try {
			File result = File.createTempFile(prefix, null);

			result.delete();
			result.mkdir();

			return result;
		} catch (IOException e) {
		}

		return null;
	}

	/**
	 * Lists all the names of the directories in the specified location. Also
	 * sort them.
	 * 
	 * @param directory
	 *            the directory
	 * @return the list of sub-directories names
	 */
	public static List<String> listSubDirectoriesName(File directory) {
		List<String> result = new ArrayList<String>();

		File[] files = directory.listFiles(new DirectoriesFileFilter());
		for (File file : files) {
			result.add(file.getName());
		}

		Collections.sort(result);

		return result;
	}

}
