/*
Copyright 2012 Helpers
https://github.com/provirus/Helpers

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package ca.pgon.helpers.iterable;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;

import ca.pgon.helpers.exceptions.IORuntimeException;

/**
 * An iterable over a file lines. It will auto-close the file at the end of
 * file.
 * 
 * @author Simon Levesque
 * 
 */
public class FileLinesIterable implements Iterable<String>, Iterator<String> {

	private BufferedReader bufferedReader;
	private String nextLine;

	private void close() {
		try {
			bufferedReader.close();
		} catch (IOException e) {
			// This is just a close, so we can safely ignore
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean hasNext() {
		return nextLine != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<String> iterator() {
		return this;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String next() {
		String result = nextLine;
		readNextLine();
		return result;
	}

	/**
	 * The file to open.
	 * 
	 * @param file
	 *            the file
	 * @throws FileNotFoundException
	 *             FileNotFoundException
	 */
	public void openFile(File file) throws FileNotFoundException {
		bufferedReader = new BufferedReader(new FileReader(file));
		readNextLine();
	}

	/**
	 * Read and store the next line in a field. When EOF or error reading,
	 * closing the resource.
	 */
	private void readNextLine() {
		try {
			nextLine = bufferedReader.readLine();
			if (nextLine == null) {
				close();
			}
		} catch (IOException e) {
			close();
			throw new IORuntimeException(e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}

}
